; ---------------------------------------------------------------------------------------------------
; MULTIPLEXACAO DE 2 DISPLAYS DE 7 SEGMENTOS HS-5101AS3                                    03/08/2014
;
; STATUS: FUNCIONA
;
; Multiplexacao de 2 displays de 7 segmentos, HS-5101AS3 de Anodo comum, usando um 
; Shift-Register SN74LS164N para comunicacao serial entre o PIC e os displays.

; Uma interrupcao do TMR1 sera usada para multiplexar os displays a cada 10ms.
;
; O programa mostrara umaa contagem de 2 digitos hexadecimais nos dois displays de 7 segmentos.
; A contagem sera limitada ao valor 255 (0xFF) pelo simbolo "cntlim".
;
; A multiplexacao sera obtida pela interrupcao do TMR1 cujo valor inicial de contagem deve
; ser especificado no simbolo "mplxtimer". Neste programa, que usa frequencia de clock (Fosc)
; de 4MHZ, o simbolo "mplxtimer" foi inicializado com o valor 55536 (decimal) que faz
; o TMR1 estourar a cada 10 milisegundos, o que proporciona um refrescamento ideal para que o 
; display nao pisque. 
;
; A selecao dos displays sera feita atraves de seus Anodos (pino 3 e 8) que estarao conectados 
; com resistores de 1k aos coletores dos transistores BC337, cujos emissores estarao aterrados e 
; suas bases conectadas com resistores de 4K7 ao PORTB<0,1>. 
;
; O display que representa o digito de mais baixa ordem estara ligado ao PORTB<0> e o de mais alta 
; ordem ao PORTB<1>.
;
; Os segmentos dos displays serao acionados conforme o layout contido na Table-Read "charset"
; residente na memoria flash do microcontrolador.
;
; ---------------------------------------------------------------------------------------------------
;
; HARDWARE - Ver esquema eletrico no arquivo "mplx2.png" no diretorio deste projeto.
; --------
;
; Fosc ............................ Interno - 4MHZ obrigatorio
;                                   A temporizacao da multiplexacao se baseia no clock de 4MHZ.
;                                   Se outra frequencia de clock for usada, o valor do simbolo
;                                   "mplxtimer" devera ser recalculado. 
;
; RB0 ............................. CS0: a base do transistor Q1 com resistor de 4K7 
;                                   (chip select do display 0 - LSB)
;
; RB1 ............................. CS1: a base do transistor Q2 com resistor de 4K7 
;                                   (chip select do display 1 - MSB)
;
; RB3 ............................. ao pino 1 do 74SL164 ---> Serial Data Input
;
; RB5 ............................. ao pino 8 do 74SL164 ---> Clock
; 
; ---------------------------------------------------------------------------------------------------
; Conexoes do Shift Register SN74LS164N ao display HS-5101AS3:
;
; SN74LS164N                                               HS-5101AS3
; PINOS FUNCAO                                                PINOS
;   1    A  (Serial In)  ao PORTB<3>                                 
;   2    B  (Serial In)  ao +5V                                                                       
;   3    Q0 ...............................................    7                                                                     
;   4    Q1 ...............................................    6
;   5    Q2 ...............................................    4
;   6    Q3 ...............................................    2
;   7    GND Ground                                                                    
;   8    CP (clock) ao PORTB<5>                                                                      
;   9    MR (+5V)                                                                        
;  10    Q4 ...............................................    1
;  11    Q5 ...............................................    9
;  12    Q6 ...............................................   10 
;  13    Q7 ...............................................    5
;  14    VCC (+5V)                                                                    
;
; ---------------------------------------------------------------------------------------------------
;
	LIST P=16F628A
	#include <p16F628A.INC>
	errorlevel  -302   
;

	__CONFIG   _CP_OFF & _LVP_OFF & _BOREN_OFF & _MCLRE_ON & _WDT_OFF & _PWRTE_ON & _INTOSC_OSC_NOCLKOUT 
;
             #include "FAMILIAS.mac"
;
             FAMILIAS
;
; ---------------------------------------------------------------------------------------------
;  SIMBOLOS USADOS NESTE PROGRAMA
; ---------------------------------------------------------------------------------------------
cntlim       equ     .255                        ; limite de contagem
mplxtimer    equ     .55536                      ; 10ms --> tempo de refresh ideal para Fosc=4MHZ 
CLOCK        equ     4000000                     ; Fosc de 4MHZ
;
DISPLAY      equ     PORTB                       ; porta de selecao dos displays
CS0          equ     0                           ; chip select do display LSB 
CS1          equ     1                           ; chip select do display Intermediario
;
SR_TRIS      equ     TRISB                       ; I/O 
SR_PORT      equ     PORTB                       ; porta de controle do shift register 74LS164
SR_7SEGDATA  equ     3                           ; serial input data -->  ao pino 1 do 74LS164
SR_7SEGCLOCK equ     5                           ; clock --> ao pino 8 do 74LS164

; ---------------------------------------------------------------------------------------------
; RAM DO SISTEMA
; ---------------------------------------------------------------------------------------------
             UDATA      0x20 
delay_temp   RES     1 
saveW        RES     1
saveST       RES     1
anodo        RES     1
ctl          RES     1
R1           RES     1
R12          RES     1
R13          RES     1
;
; ---------------------------------------------------------------------------------------------
pwron    CODE     0x0000
         goto     inicio
intr     CODE     0x0004
         goto     intrcd
; ---------------------------------------------------------------------------------------------
MAIN   CODE

; --------------------------------------------------------------------------
; Tabela de Layout para numeros de 0 a F do display de 7 segmentos HS-5101AS3
; --------------------------------------------------------------------------
charset  ADDWF  PCL,f
         RETLW   b'11111100'     ; 0     
         RETLW   b'01100000'     ; 1
         RETLW   b'11011010'     ; 2
         RETLW   b'11110010'     ; 3
         RETLW   b'01100110'     ; 4
         RETLW   b'10110110'     ; 5
         RETLW   b'10111110'     ; 6
         RETLW   b'11100000'     ; 7
         RETLW   b'11111110'     ; 8
         RETLW   b'11100110'     ; 9
         RETLW   b'11101110'     ; A
         RETLW   b'00111110'     ; b
         RETLW   b'10011100'     ; C
         RETLW   b'01111010'     ; d
         RETLW   b'11011110'     ; e
         RETLW   b'10001110'     ; F
; SEGMENTOS        ABCDEFGD
;                         P
;



; ---------------------------------------------------------------------------------------------
inicio    movlw   0x07
          movwf   CMCON                ; desliga comparadores

; --------------------------------------------------------------------------
          clrf    PORTA                          ; limpa a porta 
          clrf    PORTB                           ; limpa a porta 

          bsf     STATUS,RP0                     ; banco 1
          movlw   0xFF                           ; mascara de I/O
          movwf   TRISA                          ; configura I/O
          movlw   0xD4                           ; mascara de I/O
          movwf   TRISB                          ; configura I/O
          bcf     STATUS,RP0                     ; banco 0

          clrf    anodo                          ; limpa anodos
          bsf     anodo,0                        ; seleciona anodo do display 0
; --------------------------------------------------------------------------
; CONFIGURACAO DO TIMER 1 PARA TEMPORIZACAO DA MULTIPLEXACAO DOS DISPLAYS
; --------------------------------------------------------------------------
          bcf     T1CON,TMR1ON                   ; desliga o TIMER 
          movlw   HIGH(mplxtimer)                ; MSB para TMR1 
          movwf   TMR1H                          ; 
          movlw   LOW(mplxtimer)                 ; LSB para TMR1 
          movwf   TMR1L                          ; 
          clrf    T1CON                          ; limpa T1CON 
          bcf     T1CON,T1CKPS1                  ; prescaler 
          bcf     T1CON,T1CKPS0                  ; prescaler 
          bcf     T1CON,TMR1CS                   ; modo temporizador 
          bsf     T1CON,TMR1ON                   ; liga TIMEWK1 
          bsf     STATUS,RP0                     ; banco 1
          bsf     PIE1,TMR1IE                    ; habilita interrupcao de TIMEWK1
          bcf     STATUS,RP0                     ; banco 0
          bsf     INTCON,PEIE                    ; interrupcoes perifericas 
          bsf     INTCON,GIE                     ; habilita geral
; --------------------------------------------------------------------------
; CONTAGEM - ROTINA PRINCIPAL
;
; O programa permanece nesta rotina que sera interrompida pelo TMR1
; que mostrara o valor do contador, em hexadecimal, nos dois displays
; de 7 segmentos.
;
; --------------------------------------------------------------------------
reconta   clrf    ctl                            ; limpa o contador
soma      incf    ctl,f                          ; ctl = ctl + 1
          movlw   cntlim                         ; pega o limite
          addlw   .1                             ; compensa o limite
          xorwf   ctl,w                          ; verifica limite
          btfss   STATUS,Z                       ; limite atingido???
          goto    continua                       ; NAO --> continua contando
          goto    reconta                        ; SIM --> reinicia contagem
continua  movlw   .250                           ; tempo em milisegundos
          call    delay_ms                       ; tempo em milisegundos
          movlw   .250                           ; tempo em milisegundos
          call    delay_ms                       ; tempo em milisegundos
          goto    soma                           ; vai incrementar contador
; --------------------------------------------------------------------------
; ROTINA DE INTERRUPCAO 
; --------------------------------------------------------------------------
intrcd    bsf     STATUS,RP0                      ; banco 1
          bcf     PIE1,TMR1IE                     ; desabilita interrupcao de TIMER
          bcf     STATUS,RP0                      ; banco 0
          swapf   saveW,f                         ; salva W
          movf    STATUS,w                        ; pega o STATUS
          movwf   saveST                          ; salva o STATUS;;;;;

          btfss   anodo,0                         ; chagou a vez do anodo 0???
          goto    vanodo1                         ; NAO --> verifica anodo 2
          movf    ctl,w                           ; SIM --> pega o contador
          andlw   0xF0                            ; deixa somente o MSB
          movwf   R1                              ; salva em R1
          rrf     R1,f                            ; empurra 1 bit para a direita
          rrf     R1,f                            ; empurra 1 bit para a direita
          rrf     R1,f                            ; empurra 1 bit para a direita
          rrf     R1,f                            ; empurra 1 bit para a direita
          movlw   0x0F                            ; zera os 4 bits da esquerda
          andwf   R1,w                            ; carrega o endereco do layout
          call    charset                         ; W = layout do caracter MSB
          call    send                            ; envia o caracter ao display
          bcf     DISPLAY,CS0                     ; desabilita o display 0 (LSB)
          bsf     DISPLAY,CS1                     ; habilita o display 1 (MSB)
          clrf    anodo                           ; limpa anodo
          bsf     anodo,1                         ; arma o anodo do display 1 (MSB)
          goto    intfim                          ; fim

vanodo1   btfss   anodo,1                         ; chagou a vez do anodo 1???
          goto    intfim                          ; NAO --> sai da interrupcao
          movf    ctl,w                           ; SIM --> pega o contador
          andlw   0x0F                            ; deixa somente o LSB
          call    charset                         ; W = layout do caracter LSB
          call    send                            ; envia o caracter ao display
          bsf     DISPLAY,CS0                     ; habilita o display 0 (LSB)
          bcf     DISPLAY,CS1                     ; desabilita o display 1 (MSB)
          clrf    anodo                           ; limpa anodo
          bsf     anodo,0                         ; arma o anodo do display 0 (LSB)

intfim    movlw   HIGH(mplxtimer)                 ; MSB do TIMER1
          movwf   TMR1H                           ; recarrega MSB do TIMER1
          movlw   LOW(mplxtimer)                  ; LSB do TIMER1
          movwf   TMR1L                           ; recarrega LSB do TIMER1
          movf    saveST,w                        ; velho STATUS
          movwf   STATUS                          ; restaura o STATUS
          swapf   saveW,w                         ; restaura W
          bcf     PIR1,TMR1IF                     ; limpa flag de interrupcao
          bsf     STATUS,RP0                      ; banco 1
          bsf     PIE1,TMR1IE                     ; reabilita interrupcao de TIMER1
          bcf     STATUS,RP0                      ; banco 0
          retfie                                  ; fim

; --------------------------------------------------------------------------
; Envia, ao shift register 74LS164, 8 bits do layout do caracter.
;
; Quando o shift register 74LS164 estiver carregado com o layout do caracter, 
; ele aparecera automaticamente no display selecionado (CS0 ou CS1).
;
; ENTRADA: W contem o layout do caracter
; SAIDA  : shift register 74LS164 carregado
; --------------------------------------------------------------------------
send     movwf    R13                            ; salva o layout do caracter
         movlw    .8                             ; numero de bitas a enviar
         movwf    R12                            ; carrega o contador de bits
         bcf      SR_TRIS,SR_7SEGCLOCK           ; config I/O do sinal de clock
         bcf      SR_TRIS,SR_7SEGDATA            ; config I/O do sinal de dados
         bcf      SR_PORT,SR_7SEGCLOCK           ; abaixa o sinal de clock
send1    bcf      SR_PORT,SR_7SEGDATA            ; abaixa o sinal de dados 
         rrf     R13,f                           ; move o bit mais a direita para o carry
         btfss    STATUS,C                       ; Carry == 1 ???
         goto     strobe                         ; NAO --> envia zero
         bsf      SR_PORT,SR_7SEGDATA            ; SIM --> sobe o sinal de dados
         nop                                     ; tempo
strobe   bsf      SR_PORT,SR_7SEGCLOCK           ; levanta o sinal de clock
         nop                                     ; tempo com clock levantado
         nop                                     ; tempo com clock levantado
         nop                                     ; tempo com clock levantado
         nop                                     ; tempo com clock levantado
         nop                                     ; tempo com clock levantado
         bcf      SR_PORT,SR_7SEGCLOCK           ; abaixa o sinal de clock
         nop                                     ; tempo com clock abaixado
         nop                                     ; tempo com clock abaixado
         nop                                     ; tempo com clock abaixado
         nop                                     ; tempo com clock abaixado
         decfsz   R12,f                          ; conta bit enviado
         goto     send1                          ; se nao acabou, envia o proximo
         return                                  ; acabou --> fim
;
; --------------------------------------------------------------------------
;
         #include "delay_us.inc"
         #include "delay_ms.inc"
;


         END